/*******************************************************************************
 *   Copyright (C) 2007 by Francesco Carsana                                   *
 *   francescocarsana@ds4.it                                                   *
 *                                                                             *
 ******************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <barcode.h>


#define round(x) ((x)>=0?(long)((x)+0.5):(long)((x)-0.5))


//------------------------------------------------------------------------------

int Barcode_plt_print(struct Barcode_Item *bc, FILE *f)
{
   int i, j, barlen;
   int mode = '-'; /* text below bars */
   double scalef=1, xpos, x0, y0, yr;
   unsigned char *ptr;
   
   if (!bc->partial || !bc->textinfo) {
      bc->error = EINVAL;
      return -1;
   }
   
   /* First, calculate barlen */
   barlen = bc->partial[0] - '0';
   for (ptr = bc->partial+1; *ptr; ptr++) {
      if (isdigit(*ptr)) 
         barlen += (*ptr - '0');
      else if (islower(*ptr))
         barlen += (*ptr - 'a'+1);
   }
   
   /* The scale factor depends on bar length */
   if (!bc->scalef) {
      if (!bc->width)
         bc->width = barlen; /* default */
      scalef = bc->scalef = (double)bc->width / (double)barlen;
   }
   
   /* The width defaults to "just enough" */
   if (!bc->width)
      bc->width = barlen * scalef +1;
   
   /* But it can be too small, in this case enlarge and center the area */
   if (bc->width < barlen * scalef) {
      int wid = barlen * scalef + 1;
      bc->xoff -= (wid - bc->width)/2 ;
      bc->width = wid;
      /* Can't extend too far on the left */
      if (bc->xoff < 0) {
         bc->width += -bc->xoff;
         bc->xoff = 0;
      }
   }
   
   /* The height defaults to 80 points (rescaled) */
   if (!bc->height)
      bc->height = 80 * scalef;
   
   /* If too small (5 + text), reduce the scale factor and center */
   i = 5 + 10 * ((bc->flags & BARCODE_NO_ASCII)==0);
   if (bc->height < i * scalef ) {
      double scaleg = ((double)bc->height) / i;
      int wid = bc->width * scaleg / scalef;
      bc->xoff += (bc->width - wid)/2;
      bc->width = wid;
      scalef = scaleg;
   }
   
   /*
    * Ok, then deal with actual HPGL output
    */
   
   fprintf(f, "SP 1;\n");
   fprintf(f, "PU;\n");

   xpos = bc->margin + (bc->partial[0]-'0') * scalef;
   for (ptr = bc->partial+1, i=1; *ptr; ptr++, i++) {
      /* special cases: '+' and '-' */
      if (*ptr == '+' || *ptr == '-') {
         mode = *ptr;
         /* don't count it */
         i++;
         continue;
      }
      
      /* j is the width of this bar/space */
      if (isdigit (*ptr))
         j = *ptr-'0';
      else
         j = *ptr-'a'+1;
      
      if (i%2) { /* bar */
         x0 = bc->xoff + xpos;
         y0 = bc->yoff + bc->margin;
         yr = bc->height;
         if (!(bc->flags & BARCODE_NO_ASCII)) { /* leave space for text */
            if (mode == '-') {
               /* text below bars: 10 points or five points */
               yr -= (isdigit(*ptr) ? 10 : 5) * scalef;
            } else { /* '+' */
               /* text above bars: 10 or 0 from bottom, and 10 from top */
               y0 -= (isdigit(*ptr) ? 10 : 0) * scalef;
               yr -= (isdigit(*ptr) ? 20 : 10) * scalef; 
            }
         }
            
         int w;
//          for (w = 0; w < round(j * scalef); w++) {
//             fprintf(f,"PA %d,%d;\n", round(x0 + w), round(y0));
//             fprintf(f,"PD;\n");
//             fprintf(f,"PA %d,%d;\n", round(x0 + w), round(y0 - yr));
//             fprintf(f,"PU;\n");
//          }
         
         fprintf(f,"PA %d,%d;\n", round(x0), round(y0));
         fprintf(f,"PD;\n");
         for (w = 0; w < round(j * scalef); w+=2) {
            fprintf(f,"PA %d,%d;\n", round(x0 + w), round(y0));
            fprintf(f,"PA %d,%d;\n", round(x0 + w), round(y0 - yr));
            if (w == (round(j * scalef) - 1))
               continue;
            fprintf(f,"PA %d,%d;\n", round(x0 + w) + 1, round(y0 - yr));
            fprintf(f,"PA %d,%d;\n", round(x0 + w) + 1, round(y0));
         }
         fprintf(f,"PU;\n");
      }
      
      xpos += j * scalef;
   }
   
   /* Then, the text */
   mode = '-'; /* reinstantiate default */
   if (!(bc->flags & BARCODE_NO_ASCII)) {
//         fprintf(f, "[\n%%   char    xpos   ypos fontsize\n");
//         k=0; /* k is the "previous font size" */
//         for (ptr = bc->textinfo; ptr; ptr = strchr(ptr, ' ')) {
//             while (*ptr == ' ')
//                 ptr++;
//             if (!*ptr)
//                 break;
//             if (*ptr == '+' || *ptr == '-') {
//                 mode = *ptr;
//                 continue;
//             }
//             if (sscanf(ptr, "%lf:%lf:%c", &f1, &f2, &c) != 3) {
//                 fprintf(stderr, "barcode: impossible data: %s\n", ptr);
//                 continue;
//             }
//             
//             fprintf(f, "    [(");
//             /* Both the backslash and the two parens are special */
//             if (c=='\\' || c==')' || c=='(')
//                 fprintf(f, "\\%c) ", c);
//             else
//                 fprintf(f, "%c)  ", c);
//                 fprintf(f, "%6.2f %6.2f %5.2f]\n", 
//                 bc->xoff + f1 * scalef + bc->margin,
//                 mode == '-'
//                 ? (double)bc->yoff + bc->margin : (double)bc->yoff + bc->margin+bc->height - 8*scalef,
//                 fsav == f2 ? 0.0 : f2 * scalef);
//                 fsav = f2;
//         }
//         fprintf(f,"]   { {} forall dup 0.00 ne {\n\t"        "/Helvetica findfont exch scalefont setfont\n"        "    } {pop} ifelse\n"        "    moveto show} bind forall\n");
   }
   
   return 0;
}

//------------------------------------------------------------------------------

int main(int argc, char **argv)
{
   struct Barcode_Item* bc;
   char *line;
   FILE *file;
   
   if (argc < 2) {
      line = malloc(20);
      
      strcpy(line, "123456789012");  //BARCODE_EAN
//       strcpy(line, "1234567890128");  //BARCODE_EAN
//       strcpy(line, "123456789012 12");  //BARCODE_EAN
//       strcpy(line, "123456789012 12345");  //BARCODE_EAN
//       strcpy(line, "1234567");  //BARCODE_EAN
//       strcpy(line, "12345670 12345");  //BARCODE_EAN

//       strcpy(line, "01234567890");  //BARCODE_UPC
//       strcpy(line, "012345678905");  //BARCODE_UPC
//       strcpy(line, "012345");  //BARCODE_UPC
//       strcpy(line, "01234567890 12");  //BARCODE_UPC
//       strcpy(line, "01234567890 12345");  //BARCODE_UPC
//       strcpy(line, "0123456 12");  //BARCODE_UPC

//       strcpy(line, "1-56592-292-1");  //BARCODE_ISBN
//       strcpy(line, "3-89721-122-X");  //BARCODE_ISBN
//       strcpy(line, "3-89721-122-X 06900");  //BARCODE_ISBN

//       strcpy(line, "12345+-$");  //BARCODE_CBR

//       strcpy(line, "12345678900123456789");  //BARCODE_128        
   }
   else {
      line = malloc(strlen(argv[1]));
      strcpy(line, argv[1]);
   }
   
   bc = Barcode_Create(line);
   if (!bc) {
      fprintf(stderr, "%s: Barcode_Create(): %s\n", argv[0], strerror(errno));
      exit(1);
   }
   char ascii[strlen(bc->ascii)];
   strcpy(ascii, bc->ascii);
   printf("ascii: %s\n", ascii);
   
   Barcode_Position(bc, 1024, 0, 0, 0, 0);

   if (Barcode_Encode(bc, BARCODE_EAN) < 0) {
//    if (Barcode_Encode(bc, BARCODE_UPC) < 0) {
//    if (Barcode_Encode(bc, BARCODE_ISBN) < 0) {
//    if (Barcode_Encode(bc, BARCODE_128B) < 0) {
//    if (Barcode_Encode(bc, BARCODE_128C) < 0) {
//    if (Barcode_Encode(bc, BARCODE_128) < 0) {
//    if (Barcode_Encode(bc, BARCODE_128RAW) < 0) {
//    if (Barcode_Encode(bc, BARCODE_39) < 0) {
//    if (Barcode_Encode(bc, BARCODE_I25) < 0) {
//    if (Barcode_Encode(bc, BARCODE_CBR) < 0) {
//    if (Barcode_Encode(bc, BARCODE_MSI) < 0) {
//    if (Barcode_Encode(bc, BARCODE_PLS) < 0) {
//    if (Barcode_Encode(bc, BARCODE_93) < 0) {
//    if (Barcode_Encode(bc, BARCODE_UPC) < 0) {
      fprintf(stderr, "%s: can't encode \"%s\": %s\n", argv[0], line, strerror(bc->error));
   }
   
   char partial[strlen(bc->partial)];
   strcpy(partial, bc->partial);
   printf("partial: %s\n", partial);
   
   char textinfo[strlen(bc->textinfo)];
   strcpy(textinfo, bc->textinfo);
   printf("textinfo: %s\n", textinfo);
   
   char encoding[strlen(bc->encoding)];
   strcpy(encoding, bc->encoding);
   printf("encoding: %s\n", encoding);
   
   file = fopen("prova.plt","w");
   if (!file) {
      fprintf(stderr, "%s: %s: %s\n", argv[0], "prova.plt", strerror(errno));
      exit(1);
   }
   
   Barcode_plt_print(bc, file);
   
   printf("width: %d\n", bc->width);
   printf("height: %d\n", bc->height);
   printf("xoff: %d\n", bc->xoff);
   printf("yoff: %d\n", bc->yoff);
   printf("margin: %d\n", bc->margin);
   printf("scalef: %f\n", bc->scalef);
   printf("error: %d\n", bc->error);
   
   fclose(file);
   
   Barcode_Delete(bc);
   free(line);
   
   return 0;
}
